"""Numba-accelerated statistical functions."""

__version__ = "0.0.0"


from .speedystats import sum
from .speedystats import nansum
from .speedystats import ptp
from .speedystats import percentile
from .speedystats import nanpercentile
from .speedystats import quantile
from .speedystats import nanquantile
from .speedystats import median
from .speedystats import nanmedian
from .speedystats import average
from .speedystats import mean
from .speedystats import nanmean
from .speedystats import std
from .speedystats import nanstd
from .speedystats import var
from .speedystats import nanvar



from typing import Tuple
import numba as nb
import numpy as np


def get_nansum(data: np.ndarray, keep_axes: Tuple[int]) -> np.ndarray:
    if keep_axes == (0,):
        return numba_nansum_keep0(data)
    if keep_axes == (1,):
        return numba_nansum_keep1(data)
    if keep_axes == (2,):
        return numba_nansum_keep2(data)
    if keep_axes == (3,):
        return numba_nansum_keep3(data)
    if keep_axes == (4,):
        return numba_nansum_keep4(data)
    if keep_axes == (0, 1):
        return numba_nansum_keep01(data)
    if keep_axes == (0, 2):
        return numba_nansum_keep02(data)
    if keep_axes == (0, 3):
        return numba_nansum_keep03(data)
    if keep_axes == (0, 4):
        return numba_nansum_keep04(data)
    if keep_axes == (1, 2):
        return numba_nansum_keep12(data)
    if keep_axes == (1, 3):
        return numba_nansum_keep13(data)
    if keep_axes == (1, 4):
        return numba_nansum_keep14(data)
    if keep_axes == (2, 3):
        return numba_nansum_keep23(data)
    if keep_axes == (2, 4):
        return numba_nansum_keep24(data)
    if keep_axes == (3, 4):
        return numba_nansum_keep34(data)
    if keep_axes == (0, 1, 2):
        return numba_nansum_keep012(data)
    if keep_axes == (0, 1, 3):
        return numba_nansum_keep013(data)
    if keep_axes == (0, 1, 4):
        return numba_nansum_keep014(data)
    if keep_axes == (0, 2, 3):
        return numba_nansum_keep023(data)
    if keep_axes == (0, 2, 4):
        return numba_nansum_keep024(data)
    if keep_axes == (0, 3, 4):
        return numba_nansum_keep034(data)
    if keep_axes == (1, 2, 3):
        return numba_nansum_keep123(data)
    if keep_axes == (1, 2, 4):
        return numba_nansum_keep124(data)
    if keep_axes == (1, 3, 4):
        return numba_nansum_keep134(data)
    if keep_axes == (2, 3, 4):
        return numba_nansum_keep234(data)
    if keep_axes == (0, 1, 2, 3):
        return numba_nansum_keep0123(data)
    if keep_axes == (0, 1, 2, 4):
        return numba_nansum_keep0124(data)
    if keep_axes == (0, 1, 3, 4):
        return numba_nansum_keep0134(data)
    if keep_axes == (0, 2, 3, 4):
        return numba_nansum_keep0234(data)
    if keep_axes == (1, 2, 3, 4):
        return numba_nansum_keep1234(data)
    raise ValueError(f"Invalid data shape for nansum, received: {keep_axes}")


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep0(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0,)"""
    output = np.zeros((data.shape[0]))
    for n0 in nb.prange(data.shape[0]):
        output[n0] = np.nansum(data[n0])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep1(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (1,)"""
    output = np.zeros((data.shape[1]))
    for n0 in nb.prange(data.shape[1]):
        output[n0] = np.nansum(data[:, n0])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep2(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (2,)"""
    output = np.zeros((data.shape[2]))
    for n0 in nb.prange(data.shape[2]):
        output[n0] = np.nansum(data[:, :, n0])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep3(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (3,)"""
    output = np.zeros((data.shape[3]))
    for n0 in nb.prange(data.shape[3]):
        output[n0] = np.nansum(data[:, :, :, n0])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep4(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (4,)"""
    output = np.zeros((data.shape[4]))
    for n0 in nb.prange(data.shape[4]):
        output[n0] = np.nansum(data[:, :, :, :, n0])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep01(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 1)"""
    output = np.zeros((data.shape[0], data.shape[1]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[1]):
            output[n0, n1] = np.nansum(data[n0, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep02(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 2)"""
    output = np.zeros((data.shape[0], data.shape[2]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[2]):
            output[n0, n1] = np.nansum(data[n0, :, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep03(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 3)"""
    output = np.zeros((data.shape[0], data.shape[3]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[3]):
            output[n0, n1] = np.nansum(data[n0, :, :, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep04(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 4)"""
    output = np.zeros((data.shape[0], data.shape[4]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[4]):
            output[n0, n1] = np.nansum(data[n0, :, :, :, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep12(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (1, 2)"""
    output = np.zeros((data.shape[1], data.shape[2]))
    for n0 in nb.prange(data.shape[1]):
        for n1 in nb.prange(data.shape[2]):
            output[n0, n1] = np.nansum(data[:, n0, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep13(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (1, 3)"""
    output = np.zeros((data.shape[1], data.shape[3]))
    for n0 in nb.prange(data.shape[1]):
        for n1 in nb.prange(data.shape[3]):
            output[n0, n1] = np.nansum(data[:, n0, :, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep14(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (1, 4)"""
    output = np.zeros((data.shape[1], data.shape[4]))
    for n0 in nb.prange(data.shape[1]):
        for n1 in nb.prange(data.shape[4]):
            output[n0, n1] = np.nansum(data[:, n0, :, :, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep23(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (2, 3)"""
    output = np.zeros((data.shape[2], data.shape[3]))
    for n0 in nb.prange(data.shape[2]):
        for n1 in nb.prange(data.shape[3]):
            output[n0, n1] = np.nansum(data[:, :, n0, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep24(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (2, 4)"""
    output = np.zeros((data.shape[2], data.shape[4]))
    for n0 in nb.prange(data.shape[2]):
        for n1 in nb.prange(data.shape[4]):
            output[n0, n1] = np.nansum(data[:, :, n0, :, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep34(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (3, 4)"""
    output = np.zeros((data.shape[3], data.shape[4]))
    for n0 in nb.prange(data.shape[3]):
        for n1 in nb.prange(data.shape[4]):
            output[n0, n1] = np.nansum(data[:, :, :, n0, n1])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep012(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 1, 2)"""
    output = np.zeros((data.shape[0], data.shape[1], data.shape[2]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[1]):
            for n2 in nb.prange(data.shape[2]):
                output[n0, n1, n2] = np.nansum(data[n0, n1, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep013(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 1, 3)"""
    output = np.zeros((data.shape[0], data.shape[1], data.shape[3]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[1]):
            for n2 in nb.prange(data.shape[3]):
                output[n0, n1, n2] = np.nansum(data[n0, n1, :, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep014(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 1, 4)"""
    output = np.zeros((data.shape[0], data.shape[1], data.shape[4]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[1]):
            for n2 in nb.prange(data.shape[4]):
                output[n0, n1, n2] = np.nansum(data[n0, n1, :, :, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep023(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 2, 3)"""
    output = np.zeros((data.shape[0], data.shape[2], data.shape[3]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[2]):
            for n2 in nb.prange(data.shape[3]):
                output[n0, n1, n2] = np.nansum(data[n0, :, n1, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep024(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 2, 4)"""
    output = np.zeros((data.shape[0], data.shape[2], data.shape[4]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[2]):
            for n2 in nb.prange(data.shape[4]):
                output[n0, n1, n2] = np.nansum(data[n0, :, n1, :, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep034(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 3, 4)"""
    output = np.zeros((data.shape[0], data.shape[3], data.shape[4]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[3]):
            for n2 in nb.prange(data.shape[4]):
                output[n0, n1, n2] = np.nansum(data[n0, :, :, n1, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep123(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (1, 2, 3)"""
    output = np.zeros((data.shape[1], data.shape[2], data.shape[3]))
    for n0 in nb.prange(data.shape[1]):
        for n1 in nb.prange(data.shape[2]):
            for n2 in nb.prange(data.shape[3]):
                output[n0, n1, n2] = np.nansum(data[:, n0, n1, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep124(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (1, 2, 4)"""
    output = np.zeros((data.shape[1], data.shape[2], data.shape[4]))
    for n0 in nb.prange(data.shape[1]):
        for n1 in nb.prange(data.shape[2]):
            for n2 in nb.prange(data.shape[4]):
                output[n0, n1, n2] = np.nansum(data[:, n0, n1, :, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep134(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (1, 3, 4)"""
    output = np.zeros((data.shape[1], data.shape[3], data.shape[4]))
    for n0 in nb.prange(data.shape[1]):
        for n1 in nb.prange(data.shape[3]):
            for n2 in nb.prange(data.shape[4]):
                output[n0, n1, n2] = np.nansum(data[:, n0, :, n1, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep234(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (2, 3, 4)"""
    output = np.zeros((data.shape[2], data.shape[3], data.shape[4]))
    for n0 in nb.prange(data.shape[2]):
        for n1 in nb.prange(data.shape[3]):
            for n2 in nb.prange(data.shape[4]):
                output[n0, n1, n2] = np.nansum(data[:, :, n0, n1, n2])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep0123(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 1, 2, 3)"""
    output = np.zeros((data.shape[0], data.shape[1], data.shape[2], data.shape[3]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[1]):
            for n2 in nb.prange(data.shape[2]):
                for n3 in nb.prange(data.shape[3]):
                    output[n0, n1, n2, n3] = np.nansum(data[n0, n1, n2, n3])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep0124(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 1, 2, 4)"""
    output = np.zeros((data.shape[0], data.shape[1], data.shape[2], data.shape[4]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[1]):
            for n2 in nb.prange(data.shape[2]):
                for n3 in nb.prange(data.shape[4]):
                    output[n0, n1, n2, n3] = np.nansum(data[n0, n1, n2, :, n3])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep0134(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 1, 3, 4)"""
    output = np.zeros((data.shape[0], data.shape[1], data.shape[3], data.shape[4]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[1]):
            for n2 in nb.prange(data.shape[3]):
                for n3 in nb.prange(data.shape[4]):
                    output[n0, n1, n2, n3] = np.nansum(data[n0, n1, :, n2, n3])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep0234(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (0, 2, 3, 4)"""
    output = np.zeros((data.shape[0], data.shape[2], data.shape[3], data.shape[4]))
    for n0 in nb.prange(data.shape[0]):
        for n1 in nb.prange(data.shape[2]):
            for n2 in nb.prange(data.shape[3]):
                for n3 in nb.prange(data.shape[4]):
                    output[n0, n1, n2, n3] = np.nansum(data[n0, :, n1, n2, n3])
    return output


@nb.njit(parallel=True, fastmath=False, cache=True)
def numba_nansum_keep1234(data: np.ndarray) -> np.ndarray:
    """Numba speedup for nansum reducing all but axes (1, 2, 3, 4)"""
    output = np.zeros((data.shape[1], data.shape[2], data.shape[3], data.shape[4]))
    for n0 in nb.prange(data.shape[1]):
        for n1 in nb.prange(data.shape[2]):
            for n2 in nb.prange(data.shape[3]):
                for n3 in nb.prange(data.shape[4]):
                    output[n0, n1, n2, n3] = np.nansum(data[:, n0, n1, n2, n3])
    return output



"""Numba-accelerated statistical functions."""

from .sum import get_sum
from .nansum import get_nansum
from .ptp import get_ptp
from .percentile import get_percentile
from .nanpercentile import get_nanpercentile
from .quantile import get_quantile
from .nanquantile import get_nanquantile
from .median import get_median
from .nanmedian import get_nanmedian
from .average import get_average
from .mean import get_mean
from .nanmean import get_nanmean
from .std import get_std
from .nanstd import get_nanstd
from .var import get_var
from .nanvar import get_nanvar



from typing import Union, Iterable, Optional
import numpy as np
from .routing import speedystat_route, get_max_dims, get_keep_axes

MAX_DIMS = get_max_dims()


def _call_speedystat(
    data: np.ndarray,
    method: str,
    axis: Optional[Union[int, Iterable[int]]] = None,
    keepdims: bool = False,
    q: Optional[float] = None,
) -> np.ndarray:
    # If the axis is None, use the numpy fallback
    if axis is None:
        return _fallback_speedystat(data, method, axis, keepdims, q)

    # Identify the shape of the data and the axes to keep
    data_ndims = data.ndim
    data_shape = data.shape
    keep_axes = get_keep_axes(axis, data_ndims)

    # If no axes are kept, use the numpy fallback
    if not keep_axes:
        return _fallback_speedystat(data, method, axis, keepdims, q)

    # If the number of axes to keep isn't supported, use the numpy fallback
    if any(k >= MAX_DIMS for k in keep_axes):
        return _fallback_speedystat(data, method, axis, keepdims, q)

    # Reshape the data to be flattened along reducing axes
    last_axis = keep_axes[-1]
    if data_ndims > last_axis + 1:
        new_shape = data_shape[: last_axis + 1] + (-1,)
        data = np.reshape(data, new_shape)

    # Get the numba implementation and check if it has a q parameter
    func, has_q_param = speedystat_route(method)

    # Call the numba implementation
    if has_q_param:
        out = func(data, keep_axes, q)
    else:
        out = func(data, keep_axes)

    # Reshape the output to match the original data shape if keepdims is True
    if keepdims:
        out = np.expand_dims(out, axis)

    return out


def _fallback_speedystat(
    data: np.ndarray,
    method: str,
    axis: Optional[Union[int, Iterable[int]]] = None,
    keepdims: bool = False,
    q: Optional[float] = None,
) -> np.ndarray:
    np_method = getattr(np, method)
    if q is not None:
        return np_method(data, axis=axis, keepdims=keepdims, q=q)
    else:
        return np_method(data, axis=axis, keepdims=keepdims)


def sum(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "sum", axis, keepdims)


def nansum(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "nansum", axis, keepdims)


def ptp(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "ptp", axis, keepdims)


def percentile(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
    q: Optional[float] = None,
) -> np.ndarray:
    return _call_speedystat(data, "percentile", axis, keepdims, q)


def nanpercentile(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
    q: Optional[float] = None,
) -> np.ndarray:
    return _call_speedystat(data, "nanpercentile", axis, keepdims, q)


def quantile(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
    q: Optional[float] = None,
) -> np.ndarray:
    return _call_speedystat(data, "quantile", axis, keepdims, q)


def nanquantile(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
    q: Optional[float] = None,
) -> np.ndarray:
    return _call_speedystat(data, "nanquantile", axis, keepdims, q)


def median(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "median", axis, keepdims)


def nanmedian(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "nanmedian", axis, keepdims)


def average(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "average", axis, keepdims)


def mean(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "mean", axis, keepdims)


def nanmean(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "nanmean", axis, keepdims)


def std(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "std", axis, keepdims)


def nanstd(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "nanstd", axis, keepdims)


def var(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "var", axis, keepdims)


def nanvar(
    data: np.ndarray,
    axis: Union[int, Iterable[int]] = None,
    keepdims: bool = False,
) -> np.ndarray:
    return _call_speedystat(data, "nanvar", axis, keepdims)



from typing import Callable, Union, Iterable, Tuple
from . import numba


def speedystat_route(np_method: str) -> Callable:
    """Route numpy method names to their fast implementations.

    Args:
        np_method: Name of the numpy method to route

    Returns:
        Callable: The corresponding fast implementation
    """
    method_map = {
        "sum": numba.sum.get_sum,
        "nansum": numba.nansum.get_nansum,
        "ptp": numba.ptp.get_ptp,
        "percentile": numba.percentile.get_percentile,
        "nanpercentile": numba.nanpercentile.get_nanpercentile,
        "quantile": numba.quantile.get_quantile,
        "nanquantile": numba.nanquantile.get_nanquantile,
        "median": numba.median.get_median,
        "nanmedian": numba.nanmedian.get_nanmedian,
        "average": numba.average.get_average,
        "mean": numba.mean.get_mean,
        "nanmean": numba.nanmean.get_nanmean,
        "std": numba.std.get_std,
        "nanstd": numba.nanstd.get_nanstd,
        "var": numba.var.get_var,
        "nanvar": numba.nanvar.get_nanvar,
    }
    has_q_param = {
        "sum": False,
        "nansum": False,
        "ptp": False,
        "percentile": True,
        "nanpercentile": True,
        "quantile": True,
        "nanquantile": True,
        "median": False,
        "nanmedian": False,
        "average": False,
        "mean": False,
        "nanmean": False,
        "std": False,
        "nanstd": False,
        "var": False,
        "nanvar": False,
    }

    if np_method not in method_map:
        raise ValueError(f"No fast implementation available for {np_method}")
    if np_method not in has_q_param:
        raise ValueError(f"No q param config available for {np_method}")
    return method_map[np_method], has_q_param[np_method]


def get_max_dims() -> int:
    """Get the maximum number of dimensions supported by the fast implementations.

    Returns:
        int: Maximum number of dimensions supported
    """
    return 5


def get_keep_axes(axis: Union[int, Iterable[int]], ndim: int) -> Tuple[int]:
    keep_axes = list(range(ndim))
    if isinstance(axis, int):
        axis = [axis]
    for a in axis:
        keep_axes.remove(a)

    keep_axes = tuple(keep_axes)
    return keep_axes
